@@ -1,13 +1,13 @@
 /*
- * Copyright (C) 1996-2019 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2023 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
  * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
-#ifndef SQUID_ACL_H
-#define SQUID_ACL_H
+#ifndef SQUID_SRC_ACL_ACL_H
+#define SQUID_SRC_ACL_ACL_H
 
 #include "acl/forward.h"
 #include "acl/Options.h"
@@ -30,6 +30,12 @@ typedef ACL *(*Maker)(TypeName typeName);
 /// use the given ACL Maker for all ACLs of the named type
 void RegisterMaker(TypeName typeName, Maker maker);
 
+/// Validate and store the ACL key parameter for ACL types
+/// declared using "acl aclname type key argument..." declaration that
+/// require unique key values (if any) for each aclname+type combination.
+/// Key comparison is case-insensitive.
+void SetKey(SBuf &keyStorage, const char *keyParameterName, const char *newKey);
+
 } // namespace Acl
 
 /// A configurable condition. A node in the ACL expression tree.
@@ -48,6 +54,7 @@ public:
     static ACL *FindByName(const char *name);
 
     ACL();
+    ACL(ACL &&) = delete; // no copying of any kind
     virtual ~ACL();
 
     /// sets user-specified ACL name and squid.conf context
@@ -59,13 +66,10 @@ public:
     /// Updates the checklist state on match, async, and failure.
     bool matches(ACLChecklist *checklist) const;
 
-    /// \returns (linked) Options supported by this ACL
-    virtual const Acl::Options &options() { return Acl::NoOptions(); }
-
     /// configures ACL options, throwing on configuration errors
-    virtual void parseFlags();
+    void parseFlags();
 
-    /// parses node represenation in squid.conf; dies on failures
+    /// parses node representation in squid.conf; dies on failures
     virtual void parse() = 0;
     virtual char const *typeString() const = 0;
     virtual bool isProxyAuth() const;
@@ -95,6 +99,14 @@ private:
     virtual bool requiresRequest() const;
     /// whether our (i.e. shallow) match() requires checklist to have a reply
     virtual bool requiresReply() const;
+
+    // TODO: Rename to globalOptions(); these are not the only supported options
+    /// \returns (linked) 'global' Options supported by this ACL
+    virtual const Acl::Options &options() { return Acl::NoOptions(); }
+
+    /// \returns (linked) "line" Options supported by this ACL
+    /// \see ACL::options()
+    virtual const Acl::Options &lineOptions() { return Acl::NoOptions(); }
 };
 
 /// \ingroup ACLAPI
@@ -109,14 +121,17 @@ typedef enum {
 } aclMatchCode;
 
 /// \ingroup ACLAPI
-/// ACL check answer; TODO: Rename to Acl::Answer
-class allow_t
+/// ACL check answer
+namespace Acl {
+
+class Answer
 {
 public:
-    // not explicit: allow "aclMatchCode to allow_t" conversions (for now)
-    allow_t(const aclMatchCode aCode, int aKind = 0): code(aCode), kind(aKind) {}
+    // TODO: Find a good way to avoid implicit conversion (without explicitly
+    // casting every ACCESS_ argument in implicit constructor calls).
+    Answer(const aclMatchCode aCode, int aKind = 0): code(aCode), kind(aKind) {}
 
-    allow_t(): code(ACCESS_DUNNO), kind(0) {}
+    Answer() = default;
 
     bool operator ==(const aclMatchCode aCode) const {
         return code == aCode;
@@ -126,7 +141,7 @@ public:
         return !(*this == aCode);
     }
 
-    bool operator ==(const allow_t allow) const {
+    bool operator ==(const Answer allow) const {
         return code == allow.code && kind == allow.kind;
     }
 
@@ -146,16 +161,20 @@ public:
     /// See also: allowed().
     bool denied() const { return code == ACCESS_DENIED; }
 
-    /// whether there was either a default rule, a rule without any ACLs, or a
-    /// a rule with ACLs that all matched
-    bool someRuleMatched() const { return allowed() || denied(); }
+    /// whether Squid is uncertain about the allowed() or denied() answer
+    bool conflicted() const { return !allowed() && !denied(); }
 
-    aclMatchCode code; ///< ACCESS_* code
-    int kind; ///< which custom access list verb matched
+    aclMatchCode code = ACCESS_DUNNO; ///< ACCESS_* code
+
+    /// the matched custom access list verb (or zero)
+    int kind = 0;
+
+    /// whether we were computed by the "negate the last explicit action" rule
+    bool implicit = false;
 };
 
 inline std::ostream &
-operator <<(std::ostream &o, const allow_t a)
+operator <<(std::ostream &o, const Answer a)
 {
     switch (a) {
     case ACCESS_DENIED:
@@ -174,6 +193,8 @@ operator <<(std::ostream &o, const allow_t a)
     return o;
 }
 
+} // namespace Acl
+
 /// \ingroup ACLAPI
 class acl_proxy_auth_match_cache
 {
@@ -194,5 +215,5 @@ public:
 /// XXX: find a way to remove or at least use a refcounted ACL pointer
 extern const char *AclMatchedName;  /* NULL */
 
-#endif /* SQUID_ACL_H */
+#endif /* SQUID_SRC_ACL_ACL_H */
 
