@@ -1,13 +1,13 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2023 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
  * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
-#ifndef SQUID_EXTUSER_H
-#define SQUID_EXTUSER_H
+#ifndef SQUID_SRC_ACL_EXTUSER_H
+#define SQUID_SRC_ACL_EXTUSER_H
 
 #if USE_AUTH
 
@@ -21,24 +21,23 @@ class ACLExtUser : public ACL
 
 public:
     ACLExtUser(ACLData<char const *> *newData, char const *);
-    ACLExtUser (ACLExtUser const &old);
-    ACLExtUser & operator= (ACLExtUser const &rhs);
-    ~ACLExtUser();
+    ~ACLExtUser() override;
 
     /* ACL API */
-    virtual char const *typeString() const;
-    virtual void parse();
-    virtual void parseFlags();
-    virtual int match(ACLChecklist *checklist);
-    virtual SBufList dump() const;
-    virtual bool empty () const;
-    virtual ACL *clone()const;
+    char const *typeString() const override;
+    void parse() override;
+    int match(ACLChecklist *checklist) override;
+    SBufList dump() const override;
+    bool empty () const override;
 
 private:
+    /* ACL API */
+    const Acl::Options &lineOptions() override;
+
     ACLData<char const *> *data;
     char const *type_;
 };
 
 #endif /* USE_AUTH */
-#endif /* SQUID_EXTUSER_H */
+#endif /* SQUID_SRC_ACL_EXTUSER_H */
 
