@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2023 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -8,8 +8,8 @@
 
 /* DEBUG: section 93    eCAP Interface */
 
-#ifndef SQUID_ECAP_CONFIG_H
-#define SQUID_ECAP_CONFIG_H
+#ifndef SQUID_SRC_ADAPTATION_ECAP_CONFIG_H
+#define SQUID_SRC_ADAPTATION_ECAP_CONFIG_H
 
 #include "adaptation/Config.h"
 #include "adaptation/ServiceConfig.h"
@@ -26,7 +26,7 @@ class ServiceConfig: public Adaptation::ServiceConfig
 {
 public:
     // Adaptation::ServiceConfig API
-    virtual bool grokExtension(const char *name, const char *value);
+    bool grokExtension(const char *name, const char *value) override;
 
 public:
     typedef std::pair<std::string, std::string> Extension; // name=value in cfg
@@ -40,18 +40,18 @@ class Config: public Adaptation::Config
 
 public:
     Config();
-    ~Config();
+    ~Config() override;
 
-    virtual bool finalize();
+    bool finalize() override;
 
 protected:
-    virtual Adaptation::ServiceConfig *newServiceConfig() const;
+    Adaptation::ServiceConfig *newServiceConfig() const override;
 
 private:
     Config(const Config &); // not implemented
     Config &operator =(const Config &); // not implemented
 
-    virtual Adaptation::ServicePointer createService(const ServiceConfigPointer &cfg);
+    Adaptation::ServicePointer createService(const ServiceConfigPointer &cfg) override;
 };
 
 extern Config TheConfig;
@@ -59,5 +59,5 @@ extern Config TheConfig;
 } // namespace Ecap
 } // namespace Adaptation
 
-#endif /* SQUID_ECAP_CONFIG_H */
+#endif /* SQUID_SRC_ADAPTATION_ECAP_CONFIG_H */
 
