@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2023 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -8,13 +8,15 @@
 
 /* DEBUG: section 39    Peer user hash based selection */
 
-#ifndef SQUID_PEER_USERHASH_H_
-#define SQUID_PEER_USERHASH_H_
+#ifndef SQUID_SRC_PEER_USERHASH_H
+#define SQUID_SRC_PEER_USERHASH_H
 
 class CachePeer;
 class HttpRequest;
+class PeerSelector;
 
 void peerUserHashInit(void);
-CachePeer * peerUserHashSelectParent(HttpRequest * request);
+CachePeer * peerUserHashSelectParent(PeerSelector *);
+
+#endif /* SQUID_SRC_PEER_USERHASH_H */
 
-#endif /* SQUID_PEER_USERHASH_H_ */
